@@ -13,7 +13,7 @@
 static int
 pyint_as_int(PyObject * pyint, int *pint)
 {
-    int the_int = MyInt_AsInt(pyint);
+    int the_int = (int)PyLong_AsLong(pyint);
     if (the_int == -1 && PyErr_Occurred()) {
         return RET_ERROR;
     }
@@ -39,7 +39,7 @@ CTracer_intern_strings(void)
     int ret = RET_ERROR;
 
 #define INTERN_STRING(v, s)                     \
-    v = MyText_InternFromString(s);             \
+    v = PyUnicode_InternFromString(s);          \
     if (v == NULL) {                            \
         goto error;                             \
     }
@@ -133,14 +133,15 @@ indent(int n)
 
 static BOOL logging = FALSE;
 /* Set these constants to be a file substring and line number to start logging. */
-static const char * start_file = "tests/views";
-static int start_line = 27;
+static const char * start_file = "nested.py";
+static int start_line = 1;
 
 static void
-showlog(int depth, int lineno, PyObject * filename, const char * msg)
+CTracer_showlog(CTracer * self, int lineno, PyObject * filename, const char * msg)
 {
     if (logging) {
-        printf("%s%3d ", indent(depth), depth);
+        int depth = self->pdata_stack->depth;
+        printf("%x: %s%3d ", (int)self, indent(depth), depth);
         if (lineno) {
             printf("%4d", lineno);
         }
@@ -148,8 +149,8 @@ showlog(int depth, int lineno, PyObject * filename, const char * msg)
             printf("    ");
         }
         if (filename) {
-            PyObject *ascii = MyText_AS_BYTES(filename);
-            printf(" %s", MyBytes_AS_STRING(ascii));
+            PyObject *ascii = PyUnicode_AsASCIIString(filename);
+            printf(" %s", PyBytes_AS_STRING(ascii));
             Py_DECREF(ascii);
         }
         if (msg) {
@@ -159,9 +160,9 @@ showlog(int depth, int lineno, PyObject * filename, const char * msg)
     }
 }
 
-#define SHOWLOG(a,b,c,d)    showlog(a,b,c,d)
+#define SHOWLOG(l,f,m)    CTracer_showlog(self,l,f,m)
 #else
-#define SHOWLOG(a,b,c,d)
+#define SHOWLOG(l,f,m)
 #endif /* TRACE_LOG */
 
 #if WHAT_LOG
@@ -181,7 +182,7 @@ CTracer_record_pair(CTracer *self, int l1, int l2)
         goto error;
     }
 
-    if (PyDict_SetItem(self->pcur_entry->file_data, t, Py_None) < 0) {
+    if (PySet_Add(self->pcur_entry->file_data, t) < 0) {
         goto error;
     }
 
@@ -232,7 +233,7 @@ CTracer_set_pdata_stack(CTracer *self)
 
             /* A new concurrency object.  Make a new data stack. */
             the_index = self->data_stacks_used;
-            stack_index = MyInt_FromInt(the_index);
+            stack_index = PyLong_FromLong((long)the_index);
             if (stack_index == NULL) {
                 goto error;
             }
@@ -305,7 +306,7 @@ CTracer_check_missing_return(CTracer *self, PyFrameObject *frame)
                         goto error;
                     }
                 }
-                SHOWLOG(self->pdata_stack->depth, frame->f_lineno, frame->f_code->co_filename, "missedreturn");
+                SHOWLOG(PyFrame_GetLineNumber(frame), MyFrame_GetCode(frame)->co_filename, "missedreturn");
                 self->pdata_stack->depth--;
                 self->pcur_entry = &self->pdata_stack->stack[self->pdata_stack->depth];
             }
@@ -384,7 +385,7 @@ CTracer_handle_call(CTracer *self, PyFrameObject *frame)
     }
 
     /* Check if we should trace this line. */
-    filename = frame->f_code->co_filename;
+    filename = MyFrame_GetCode(frame)->co_filename;
     disposition = PyDict_GetItem(self->should_trace_cache, filename);
     if (disposition == NULL) {
         if (PyErr_Occurred()) {
@@ -503,7 +504,7 @@ CTracer_handle_call(CTracer *self, PyFrameObject *frame)
             if (PyErr_Occurred()) {
                 goto error;
             }
-            file_data = PyDict_New();
+            file_data = PySet_New(NULL);
             if (file_data == NULL) {
                 goto error;
             }
@@ -529,30 +530,30 @@ CTracer_handle_call(CTracer *self, PyFrameObject *frame)
         self->pcur_entry->file_data = file_data;
         self->pcur_entry->file_tracer = file_tracer;
 
-        SHOWLOG(self->pdata_stack->depth, frame->f_lineno, filename, "traced");
+        SHOWLOG(PyFrame_GetLineNumber(frame), filename, "traced");
     }
     else {
         Py_XDECREF(self->pcur_entry->file_data);
         self->pcur_entry->file_data = NULL;
         self->pcur_entry->file_tracer = Py_None;
-        SHOWLOG(self->pdata_stack->depth, frame->f_lineno, filename, "skipped");
+        SHOWLOG(PyFrame_GetLineNumber(frame), filename, "skipped");
     }
 
     self->pcur_entry->disposition = disposition;
 
     /* Make the frame right in case settrace(gettrace()) happens. */
     Py_INCREF(self);
-    My_XSETREF(frame->f_trace, (PyObject*)self);
+    Py_XSETREF(frame->f_trace, (PyObject*)self);
 
     /* A call event is really a "start frame" event, and can happen for
      * re-entering a generator also.  f_lasti is -1 for a true call, and a
      * real byte offset for a generator re-entry.
      */
     if (frame->f_lasti < 0) {
-        self->pcur_entry->last_line = -frame->f_code->co_firstlineno;
+        self->pcur_entry->last_line = -MyFrame_GetCode(frame)->co_firstlineno;
     }
     else {
-        self->pcur_entry->last_line = frame->f_lineno;
+        self->pcur_entry->last_line = PyFrame_GetLineNumber(frame);
     }
 
 ok:
@@ -633,7 +634,7 @@ CTracer_handle_line(CTracer *self, PyFrameObject *frame)
 
     STATS( self->stats.lines++; )
     if (self->pdata_stack->depth >= 0) {
-        SHOWLOG(self->pdata_stack->depth, frame->f_lineno, frame->f_code->co_filename, "line");
+        SHOWLOG(PyFrame_GetLineNumber(frame), MyFrame_GetCode(frame)->co_filename, "line");
         if (self->pcur_entry->file_data) {
             int lineno_from = -1;
             int lineno_to = -1;
@@ -655,7 +656,7 @@ CTracer_handle_line(CTracer *self, PyFrameObject *frame)
                 }
             }
             else {
-                lineno_from = lineno_to = frame->f_lineno;
+                lineno_from = lineno_to = PyFrame_GetLineNumber(frame);
             }
 
             if (lineno_from != -1) {
@@ -668,12 +669,12 @@ CTracer_handle_line(CTracer *self, PyFrameObject *frame)
                     }
                     else {
                         /* Tracing lines: key is simply this_line. */
-                        PyObject * this_line = MyInt_FromInt(lineno_from);
+                        PyObject * this_line = PyLong_FromLong((long)lineno_from);
                         if (this_line == NULL) {
                             goto error;
                         }
 
-                        ret2 = PyDict_SetItem(self->pcur_entry->file_data, this_line, Py_None);
+                        ret2 = PySet_Add(self->pcur_entry->file_data, this_line);
                         Py_DECREF(this_line);
                         if (ret2 < 0) {
                             goto error;
@@ -714,14 +715,14 @@ CTracer_handle_return(CTracer *self, PyFrameObject *frame)
              * f_lasti before reading the byte.
              */
             int bytecode = RETURN_VALUE;
-            PyObject * pCode = frame->f_code->co_code;
-            int lasti = frame->f_lasti;
+            PyObject * pCode = MyFrame_GetCode(frame)->co_code;
+            int lasti = MyFrame_lasti(frame);
 
-            if (lasti < MyBytes_GET_SIZE(pCode)) {
-                bytecode = MyBytes_AS_STRING(pCode)[lasti];
+            if (lasti < PyBytes_GET_SIZE(pCode)) {
+                bytecode = PyBytes_AS_STRING(pCode)[lasti];
             }
             if (bytecode != YIELD_VALUE) {
-                int first = frame->f_code->co_firstlineno;
+                int first = MyFrame_GetCode(frame)->co_firstlineno;
                 if (CTracer_record_pair(self, self->pcur_entry->last_line, -first) < 0) {
                     goto error;
                 }
@@ -744,7 +745,7 @@ CTracer_handle_return(CTracer *self, PyFrameObject *frame)
         }
 
         /* Pop the stack. */
-        SHOWLOG(self->pdata_stack->depth, frame->f_lineno, frame->f_code->co_filename, "return");
+        SHOWLOG(PyFrame_GetLineNumber(frame), MyFrame_GetCode(frame)->co_filename, "return");
         self->pdata_stack->depth--;
         self->pcur_entry = &self->pdata_stack->stack[self->pdata_stack->depth];
     }
@@ -775,7 +776,7 @@ CTracer_handle_exception(CTracer *self, PyFrameObject *frame)
     */
     STATS( self->stats.exceptions++; )
     self->last_exc_back = frame->f_back;
-    self->last_exc_firstlineno = frame->f_code->co_firstlineno;
+    self->last_exc_firstlineno = MyFrame_GetCode(frame)->co_firstlineno;
 
     return RET_OK;
 }
@@ -806,15 +807,15 @@ CTracer_trace(CTracer *self, PyFrameObject *frame, int what, PyObject *arg_unuse
 
     #if WHAT_LOG
     if (what <= (int)(sizeof(what_sym)/sizeof(const char *))) {
-        ascii = MyText_AS_BYTES(frame->f_code->co_filename);
-        printf("trace: %s @ %s %d\n", what_sym[what], MyBytes_AS_STRING(ascii), frame->f_lineno);
+        ascii = PyUnicode_AsASCIIString(MyFrame_GetCode(frame)->co_filename);
+        printf("%x trace: f:%x %s @ %s %d\n", (int)self, (int)frame, what_sym[what], PyBytes_AS_STRING(ascii), PyFrame_GetLineNumber(frame));
         Py_DECREF(ascii);
     }
     #endif
 
     #if TRACE_LOG
-    ascii = MyText_AS_BYTES(frame->f_code->co_filename);
-    if (strstr(MyBytes_AS_STRING(ascii), start_file) && frame->f_lineno == start_line) {
+    ascii = PyUnicode_AsASCIIString(MyFrame_GetCode(frame)->co_filename);
+    if (strstr(PyBytes_AS_STRING(ascii), start_file) && PyFrame_GetLineNumber(frame) == start_line) {
         logging = TRUE;
     }
     Py_DECREF(ascii);
@@ -913,7 +914,7 @@ CTracer_call(CTracer *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"frame", "event", "arg", "lineno", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!O!O|i:Tracer_call", kwlist,
-            &PyFrame_Type, &frame, &MyText_Type, &what_str, &arg, &lineno)) {
+            &PyFrame_Type, &frame, &PyUnicode_Type, &what_str, &arg, &lineno)) {
         goto done;
     }
 
@@ -921,8 +922,8 @@ CTracer_call(CTracer *self, PyObject *args, PyObject *kwds)
        for the C function. */
     for (what = 0; what_names[what]; what++) {
         int should_break;
-        ascii = MyText_AS_BYTES(what_str);
-        should_break = !strcmp(MyBytes_AS_STRING(ascii), what_names[what]);
+        ascii = PyUnicode_AsASCIIString(what_str);
+        should_break = !strcmp(PyBytes_AS_STRING(ascii), what_names[what]);
         Py_DECREF(ascii);
         if (should_break) {
             break;
@@ -930,8 +931,8 @@ CTracer_call(CTracer *self, PyObject *args, PyObject *kwds)
     }
 
     #if WHAT_LOG
-    ascii = MyText_AS_BYTES(frame->f_code->co_filename);
-    printf("pytrace: %s @ %s %d\n", what_sym[what], MyBytes_AS_STRING(ascii), frame->f_lineno);
+    ascii = PyUnicode_AsASCIIString(MyFrame_GetCode(frame)->co_filename);
+    printf("pytrace: %s @ %s %d\n", what_sym[what], PyBytes_AS_STRING(ascii), PyFrame_GetLineNumber(frame));
     Py_DECREF(ascii);
     #endif
 
@@ -1108,7 +1109,7 @@ CTracer_methods[] = {
 
 PyTypeObject
 CTracerType = {
-    MyType_HEAD_INIT
+    PyVarObject_HEAD_INIT(NULL, 0)
     "coverage.CTracer",        /*tp_name*/
     sizeof(CTracer),           /*tp_basicsize*/
     0,                         /*tp_itemsize*/
