@@ -1,4 +1,3 @@
-# coding: utf-8
 # Licensed under the Apache License: http://www.apache.org/licenses/LICENSE-2.0
 # For details: https://github.com/nedbat/coveragepy/blob/master/NOTICE.txt
 
@@ -20,9 +19,13 @@ import textwrap
 import warnings
 import zipfile
 
-import pytest
+try:
+    import pytest
+except ImportError:
+    # We want to be able to run this for some tasks that don't need pytest.
+    pytest = None
 
-# Contants derived the same as in coverage/env.py.  We can't import
+# Constants derived the same as in coverage/env.py.  We can't import
 # that file here, it would be evaluated too early and not get the
 # settings we make in this file.
 
@@ -45,7 +48,7 @@ def do_show_env():
     """Show the environment variables."""
     print("Environment:")
     for env in sorted(os.environ):
-        print("  %s = %r" % (env, os.environ[env]))
+        print(f"  {env} = {os.environ[env]!r}")
 
 
 def do_remove_extension():
@@ -89,7 +92,7 @@ def should_skip(tracer):
                 skipper = "Only one tracer: no Python tracer for CPython"
         else:
             if tracer == "c":
-                skipper = "No C tracer for {}".format(platform.python_implementation())
+                skipper = f"No C tracer for {platform.python_implementation()}"
     elif tracer == "py":
         # $set_env.py: COVERAGE_NO_PYTRACER - Don't run the tests under the Python tracer.
         skipper = os.environ.get("COVERAGE_NO_PYTRACER")
@@ -113,7 +116,7 @@ def make_env_id(tracer):
     version = "%s%s" % sys.version_info[:2]
     if PYPY:
         version += "_%s%s" % sys.pypy_version_info[:2]
-    env_id = "%s%s_%s" % (impl, version, tracer)
+    env_id = f"{impl}{version}_{tracer}"
     return env_id
 
 
@@ -121,9 +124,6 @@ def run_tests(tracer, *runner_args):
     """The actual running of tests."""
     if 'COVERAGE_TESTING' not in os.environ:
         os.environ['COVERAGE_TESTING'] = "True"
-    # $set_env.py: COVERAGE_ENV_ID - Use environment-specific test directories.
-    if 'COVERAGE_ENV_ID' in os.environ:
-        os.environ['COVERAGE_ENV_ID'] = make_env_id(tracer)
     print_banner(label_for_tracer(tracer))
 
     return pytest.main(list(runner_args))
@@ -135,6 +135,9 @@ def run_tests_with_coverage(tracer, *runner_args):
     os.environ['COVERAGE_TESTING'] = "True"
     os.environ['COVERAGE_PROCESS_START'] = os.path.abspath('metacov.ini')
     os.environ['COVERAGE_HOME'] = os.getcwd()
+    context = os.environ.get('COVERAGE_CONTEXT')
+    if context:
+        os.environ['COVERAGE_CONTEXT'] = context + "." + tracer
 
     # Create the .pth file that will let us measure coverage in sub-processes.
     # The .pth file seems to have to be alphabetically after easy-install.pth
@@ -145,7 +148,7 @@ def run_tests_with_coverage(tracer, *runner_args):
     with open(pth_path, "w") as pth_file:
         pth_file.write("import coverage; coverage.process_startup()\n")
 
-    suffix = "%s_%s" % (make_env_id(tracer), platform.platform())
+    suffix = f"{make_env_id(tracer)}_{platform.platform()}"
     os.environ['COVERAGE_METAFILE'] = os.path.abspath(".metacov."+suffix)
 
     import coverage
@@ -157,7 +160,7 @@ def run_tests_with_coverage(tracer, *runner_args):
     try:
         # Re-import coverage to get it coverage tested!  I don't understand all
         # the mechanics here, but if I don't carry over the imported modules
-        # (in covmods), then things go haywire (os == None, eventually).
+        # (in covmods), then things go haywire (os is None, eventually).
         covmods = {}
         covdir = os.path.split(coverage.__file__)[0]
         # We have to make a list since we'll be deleting in the loop.
@@ -192,7 +195,7 @@ def do_combine_html():
     cov.load()
     cov.combine()
     cov.save()
-    show_contexts = bool(os.environ.get('COVERAGE_CONTEXT'))
+    show_contexts = bool(os.environ.get('COVERAGE_DYNCTX') or os.environ.get('COVERAGE_CONTEXT'))
     cov.html_report(show_contexts=show_contexts)
     cov.xml_report()
 
@@ -213,54 +216,41 @@ def do_test_with_tracer(tracer, *runner_args):
 
 
 def do_zip_mods():
-    """Build the zipmods.zip file."""
-    zf = zipfile.ZipFile("tests/zipmods.zip", "w")
+    """Build the zip files needed for tests."""
+    with zipfile.ZipFile("tests/zipmods.zip", "w") as zf:
 
-    # Take one file from disk.
-    zf.write("tests/covmodzip1.py", "covmodzip1.py")
+        # Take some files from disk.
+        zf.write("tests/covmodzip1.py", "covmodzip1.py")
 
-    # The others will be various encodings.
-    source = textwrap.dedent(u"""\
-        # coding: {encoding}
-        text = u"{text}"
-        ords = {ords}
-        assert [ord(c) for c in text] == ords
-        print(u"All OK with {encoding}")
-        """)
-    # These encodings should match the list in tests/test_python.py
-    details = [
-        (u'utf8', u'ⓗⓔⓛⓛⓞ, ⓦⓞⓡⓛⓓ'),
-        (u'gb2312', u'你好，世界'),
-        (u'hebrew', u'שלום, עולם'),
-        (u'shift_jis', u'こんにちは世界'),
-        (u'cp1252', u'“hi”'),
-    ]
-    for encoding, text in details:
-        filename = 'encoded_{}.py'.format(encoding)
-        ords = [ord(c) for c in text]
-        source_text = source.format(encoding=encoding, text=text, ords=ords)
-        zf.writestr(filename, source_text.encode(encoding))
+        # The others will be various encodings.
+        source = textwrap.dedent("""\
+            # coding: {encoding}
+            text = u"{text}"
+            ords = {ords}
+            assert [ord(c) for c in text] == ords
+            print(u"All OK with {encoding}")
+            encoding = "{encoding}"
+            """)
+        # These encodings should match the list in tests/test_python.py
+        details = [
+            ('utf8', 'ⓗⓔⓛⓛⓞ, ⓦⓞⓡⓛⓓ'),
+            ('gb2312', '你好，世界'),
+            ('hebrew', 'שלום, עולם'),
+            ('shift_jis', 'こんにちは世界'),
+            ('cp1252', '“hi”'),
+        ]
+        for encoding, text in details:
+            filename = f'encoded_{encoding}.py'
+            ords = [ord(c) for c in text]
+            source_text = source.format(encoding=encoding, text=text, ords=ords)
+            zf.writestr(filename, source_text.encode(encoding))
 
-    zf.close()
+    with zipfile.ZipFile("tests/zip1.zip", "w") as zf:
+        zf.write("tests/zipsrc/zip1/__init__.py", "zip1/__init__.py")
+        zf.write("tests/zipsrc/zip1/zip1.py", "zip1/zip1.py")
 
-    zf = zipfile.ZipFile("tests/covmain.zip", "w")
-    zf.write("coverage/__main__.py", "__main__.py")
-    zf.close()
-
-
-def do_install_egg():
-    """Install the egg1 egg for tests."""
-    # I am pretty certain there are easier ways to install eggs...
-    cur_dir = os.getcwd()
-    os.chdir("tests/eggsrc")
-    with ignore_warnings():
-        import distutils.core
-        distutils.core.run_setup("setup.py", ["--quiet", "bdist_egg"])
-        egg = glob.glob("dist/*.egg")[0]
-        distutils.core.run_setup(
-            "setup.py", ["--quiet", "easy_install", "--no-deps", "--zip-ok", egg]
-        )
-    os.chdir(cur_dir)
+    with zipfile.ZipFile("tests/covmain.zip", "w") as zf:
+        zf.write("coverage/__main__.py", "__main__.py")
 
 
 def do_check_eol():
@@ -300,7 +290,7 @@ def do_check_eol():
                         return
 
         if line is not None and not line.strip():
-            print("%s: final blank line" % (fname,))
+            print(f"{fname}: final blank line")
 
     def check_files(root, patterns, **kwargs):
         """Check a number of files for whitespace abuse."""
@@ -348,7 +338,7 @@ def print_banner(label):
 
     rev = platform.python_revision()
     if rev:
-        version += " (rev {})".format(rev)
+        version += f" (rev {rev})"
 
     try:
         which_python = os.path.relpath(sys.executable)
@@ -356,7 +346,7 @@ def print_banner(label):
         # On Windows having a python executable on a different drive
         # than the sources cannot be relative.
         which_python = sys.executable
-    print('=== %s %s %s (%s) ===' % (impl, version, label, which_python))
+    print(f'=== {impl} {version} {label} ({which_python}) ===')
     sys.stdout.flush()
 
 
@@ -366,7 +356,7 @@ def do_help():
     items.sort()
     for name, value in items:
         if name.startswith('do_'):
-            print("%-20s%s" % (name[3:], value.__doc__))
+            print(f"{name[3:]:<20}{value.__doc__}")
 
 
 def analyze_args(function):
@@ -377,14 +367,8 @@ def analyze_args(function):
             star(boolean): Does `function` accept *args?
             num_args(int): How many positional arguments does `function` have?
     """
-    try:
-        getargspec = inspect.getfullargspec
-    except AttributeError:
-        getargspec = inspect.getargspec
-    with ignore_warnings():
-        # DeprecationWarning: Use inspect.signature() instead of inspect.getfullargspec()
-        argspec = getargspec(function)
-    return bool(argspec[1]), len(argspec[0])
+    argspec = inspect.getfullargspec(function)
+    return bool(argspec.varargs), len(argspec.args)
 
 
 def main(args):
