@@ -14,35 +14,26 @@ import warnings
 import pytest
 
 from coverage import env
+from coverage.exceptions import StopEverything
 
 
+# Pytest will rewrite assertions in test modules, but not elsewhere.
+# This tells pytest to also rewrite assertions in coveragetest.py.
+pytest.register_assert_rewrite("tests.coveragetest")
+pytest.register_assert_rewrite("tests.helpers")
+
 # Pytest can take additional options:
 # $set_env.py: PYTEST_ADDOPTS - Extra arguments to pytest.
 
 @pytest.fixture(autouse=True)
 def set_warnings():
-    """Enable DeprecationWarnings during all tests."""
+    """Configure warnings to show while running tests."""
     warnings.simplefilter("default")
     warnings.simplefilter("once", DeprecationWarning)
 
     # Warnings to suppress:
     # How come these warnings are successfully suppressed here, but not in setup.cfg??
 
-    #   setuptools/py33compat.py:54: DeprecationWarning: The value of convert_charrefs will become
-    #   True in 3.5. You are encouraged to set the value explicitly.
-    #       unescape = getattr(html, 'unescape', html_parser.HTMLParser().unescape)
-    warnings.filterwarnings(
-        "ignore",
-        category=DeprecationWarning,
-        message=r"The value of convert_charrefs will become True in 3.5.",
-        )
-
-    warnings.filterwarnings(
-        "ignore",
-        category=DeprecationWarning,
-        message=r".* instead of inspect.getfullargspec",
-        )
-
     # <frozen importlib._bootstrap>:681:
     # ImportWarning: VendorImporter.exec_module() not found; falling back to load_module()
     warnings.filterwarnings(
@@ -50,8 +41,19 @@ def set_warnings():
         category=ImportWarning,
         message=r".*exec_module\(\) not found; falling back to load_module\(\)",
         )
+    # <frozen importlib._bootstrap>:908:
+    # ImportWarning: AssertionRewritingHook.find_spec() not found; falling back to find_module()
+    # <frozen importlib._bootstrap>:908:
+    # ImportWarning: _SixMetaPathImporter.find_spec() not found; falling back to find_module()
+    # <frozen importlib._bootstrap>:908:
+    # ImportWarning: VendorImporter.find_spec() not found; falling back to find_module()
+    warnings.filterwarnings(
+        "ignore",
+        category=ImportWarning,
+        message=r".*find_spec\(\) not found; falling back to find_module\(\)",
+        )
 
-    if env.PYPY3:
+    if env.PYPY:
         # pypy3 warns about unclosed files a lot.
         warnings.filterwarnings("ignore", r".*unclosed file", category=ResourceWarning)
 
@@ -77,7 +79,7 @@ def fix_xdist_sys_path():
     See: https://github.com/pytest-dev/pytest-xdist/issues/376
 
     """
-    if os.environ.get('PYTEST_XDIST_WORKER', ''):
+    if os.environ.get('PYTEST_XDIST_WORKER', ''):       # pragma: part covered
         # We are running in an xdist worker.
         if sys.path[1] == '':
             # xdist has set sys.path[1] to ''.  Clobber it.
@@ -87,3 +89,11 @@ def fix_xdist_sys_path():
             del os.environ['PYTHONPATH']
         except KeyError:
             pass
+
+
+@pytest.hookimpl(hookwrapper=True)
+def pytest_runtest_call(item):
+    """Convert StopEverything into skipped tests."""
+    outcome = yield
+    if outcome.excinfo and issubclass(outcome.excinfo[0], StopEverything):
+        pytest.skip(f"Skipping {item.nodeid} for StopEverything: {outcome.excinfo[1]}")
