@@ -6,6 +6,7 @@
 import pytest
 
 from tests.coveragetest import CoverageTest
+from tests.helpers import assert_count_equal
 
 import coverage
 from coverage import env
@@ -94,7 +95,7 @@ class SimpleArcTest(CoverageTest):
                 if x % 2: return True
                 return False
             a = fn(1)
-            assert a == True
+            assert a is True
             """,
             arcz=".1 14 45 5.  .2 2. 23 3.", arcz_missing="23 3.")
 
@@ -160,24 +161,65 @@ class SimpleArcTest(CoverageTest):
             arcz_missing=arcz_missing,
         )
 
+    def test_bug_1184(self):
+        self.check_coverage("""\
+            def foo(x):
+                if x:
+                    try:
+                        1/(x - 1)
+                    except ZeroDivisionError:
+                        pass
+                return x        # 7
+
+            for i in range(3):  # 9
+                foo(i)
+            """,
+            arcz=".1 19 9-1  .2 23 27 34 47 56 67 7-1  9A A9",
+            arcz_unpredicted="45",
+            )
+
 
 class WithTest(CoverageTest):
     """Arc-measuring tests involving context managers."""
 
     def test_with(self):
+        if env.PYBEHAVIOR.exit_through_with:
+            arcz = ".1 .2 23 34 42 2. 16 6."
+        else:
+            arcz = ".1 .2 23 34 4. 16 6."
         self.check_coverage("""\
             def example():
-                with open("test", "w") as f: # exit
-                    f.write("")
-                    return 1
+                with open("test", "w") as f:
+                    f.write("3")
+                    a = 4
 
             example()
             """,
-            arcz=".1 .2 23 34 4. 16 6."
+            arcz=arcz,
+            )
+
+    def test_with_return(self):
+        if env.PYBEHAVIOR.exit_through_with:
+            arcz = ".1 .2 23 34 42 2. 16 6."
+        else:
+            arcz = ".1 .2 23 34 4. 16 6."
+        self.check_coverage("""\
+            def example():
+                with open("test", "w") as f:
+                    f.write("3")
+                    return 4
+
+            example()
+            """,
+            arcz=arcz,
             )
 
     def test_bug_146(self):
         # https://github.com/nedbat/coveragepy/issues/146
+        if env.PYBEHAVIOR.exit_through_with:
+            arcz = ".1 12 23 32 24 41 15 5."
+        else:
+            arcz = ".1 12 23 34 41 15 5."
         self.check_coverage("""\
             for i in range(2):
                 with open("test", "w") as f:
@@ -185,9 +227,114 @@ class WithTest(CoverageTest):
                 print(4)
             print(5)
             """,
-            arcz=".1 12 23 34 41 15 5."
+            arcz=arcz,
             )
 
+    def test_nested_with_return(self):
+        if env.PYBEHAVIOR.exit_through_with:
+            arcz = ".1 .2 23 34 45 56 64 42 2. 18 8."
+        else:
+            arcz = ".1 .2 23 34 45 56 6. 18 8."
+        self.check_coverage("""\
+            def example(x):
+                with open("test", "w") as f2:
+                    a = 3
+                    with open("test2", "w") as f4:
+                        f2.write("5")
+                        return 6
+
+            example(8)
+            """,
+            arcz=arcz,
+            )
+
+    def test_break_through_with(self):
+        if env.PYBEHAVIOR.exit_through_with:
+            arcz = ".1 12 23 34 42 25 15 5."
+        else:
+            arcz = ".1 12 23 34 45 15 5."
+        self.check_coverage("""\
+            for i in range(1+1):
+                with open("test", "w") as f:
+                    print(3)
+                    break
+            print(5)
+            """,
+            arcz=arcz,
+            arcz_missing="15",
+            )
+
+    def test_continue_through_with(self):
+        if env.PYBEHAVIOR.exit_through_with:
+            arcz = ".1 12 23 34 42 21 15 5."
+        else:
+            arcz = ".1 12 23 34 41 15 5."
+        self.check_coverage("""\
+            for i in range(1+1):
+                with open("test", "w") as f:
+                    print(3)
+                    continue
+            print(5)
+            """,
+            arcz=arcz,
+            )
+
+    def test_raise_through_with(self):
+        if env.PYBEHAVIOR.exit_through_with:
+            arcz = ".1 12 27 78 8. 9A A.  -23 34 45 53 6-2"
+            arcz_missing = "6-2 8."
+            arcz_unpredicted = "3-2 89"
+        else:
+            arcz = ".1 12 27 78 8. 9A A.  -23 34 45 5-2 6-2"
+            arcz_missing = "6-2 8."
+            arcz_unpredicted = "89"
+        cov = self.check_coverage("""\
+            from contextlib import suppress
+            def f(x):
+                with suppress():    # used as a null context manager
+                    print(4)
+                    raise Exception("Boo6")
+                print(6)
+            try:
+                f(8)
+            except Exception:
+                print("oops 10")
+            """,
+            arcz=arcz,
+            arcz_missing=arcz_missing,
+            arcz_unpredicted=arcz_unpredicted,
+            )
+        expected = "line 3 didn't jump to the function exit"
+        assert self.get_missing_arc_description(cov, 3, -2) == expected
+
+    def test_untaken_raise_through_with(self):
+        if env.PYBEHAVIOR.exit_through_with:
+            #arcz = ".1 12 28 89 9. AB B.  -23 3-2 34 45 56 53 63 37 7-2"
+            arcz = ".1 12 28 89 9. AB B.  -23 34 45 56 53 63 37 7-2"
+            #arcz_missing = "3-2 56 63 AB B."
+            arcz_missing = "56 63 AB B."
+        else:
+            arcz = ".1 12 28 89 9. AB B.  -23 34 45 56 6-2 57 7-2"
+            arcz_missing = "56 6-2 AB B."
+        cov = self.check_coverage("""\
+            from contextlib import suppress
+            def f(x):
+                with suppress():    # used as a null context manager
+                    print(4)
+                    if x == 5:
+                        raise Exception("Boo6")
+                print(7)
+            try:
+                f(9)
+            except Exception:
+                print("oops 11")
+            """,
+            arcz=arcz,
+            arcz_missing=arcz_missing,
+            )
+        expected = "line 3 didn't jump to the function exit"
+        assert self.get_missing_arc_description(cov, 3, -2) == expected
+
 
 class LoopArcTest(CoverageTest):
     """Arc-measuring tests involving loops."""
@@ -270,13 +417,10 @@ class LoopArcTest(CoverageTest):
         # With "while 1", the loop knows it's constant.
         if env.PYBEHAVIOR.keep_constant_test:
             arcz = ".1 12 23 34 45 36 62 57 7."
-            arcz_missing = ""
         elif env.PYBEHAVIOR.nix_while_true:
             arcz = ".1 13 34 45 36 63 57 7."
-            arcz_missing = ""
         else:
             arcz = ".1 12 23 34 45 36 63 57 7."
-            arcz_missing = ""
         self.check_coverage("""\
             a, i = 1, 0
             while 1:
@@ -287,7 +431,6 @@ class LoopArcTest(CoverageTest):
             assert a == 4 and i == 3
             """,
             arcz=arcz,
-            arcz_missing=arcz_missing,
             )
 
     def test_while_true(self):
@@ -297,10 +440,8 @@ class LoopArcTest(CoverageTest):
             arcz = ".1 12 23 34 45 36 62 57 7."
         elif env.PYBEHAVIOR.nix_while_true:
             arcz = ".1 13 34 45 36 63 57 7."
-        elif env.PY3:
-            arcz = ".1 12 23 34 45 36 63 57 7."
         else:
-            arcz = ".1 12 23 34 45 36 62 57 7."
+            arcz = ".1 12 23 34 45 36 63 57 7."
         self.check_coverage("""\
             a, i = 1, 0
             while True:
@@ -322,7 +463,7 @@ class LoopArcTest(CoverageTest):
                     return 1
             """)
         out = self.run_command("coverage run --branch --source=. main.py")
-        self.assertEqual(out, 'done\n')
+        assert out == 'done\n'
         if env.PYBEHAVIOR.keep_constant_test:
             num_stmts = 3
         elif env.PYBEHAVIOR.nix_while_true:
@@ -332,7 +473,7 @@ class LoopArcTest(CoverageTest):
         expected = "zero.py {n} {n} 0 0 0% 1-3".format(n=num_stmts)
         report = self.report_from_command("coverage report -m")
         squeezed = self.squeezed_lines(report)
-        self.assertIn(expected, squeezed[3])
+        assert expected in squeezed[3]
 
     def test_bug_496_continue_in_constant_while(self):
         # https://github.com/nedbat/coveragepy/issues/496
@@ -341,10 +482,8 @@ class LoopArcTest(CoverageTest):
             arcz = ".1 12 23 34 45 52 46 67 7."
         elif env.PYBEHAVIOR.nix_while_true:
             arcz = ".1 13 34 45 53 46 67 7."
-        elif env.PY3:
-            arcz = ".1 12 23 34 45 53 46 67 7."
         else:
-            arcz = ".1 12 23 34 45 52 46 67 7."
+            arcz = ".1 12 23 34 45 53 46 67 7."
         self.check_coverage("""\
             up = iter('ta')
             while True:
@@ -416,11 +555,6 @@ class LoopArcTest(CoverageTest):
         )
 
     def test_confusing_for_loop_bug_175(self):
-        if env.PY3:
-            # Py3 counts the list comp as a separate code object.
-            arcz = ".1 -22 2-2 12 23 34 45 53 3."
-        else:
-            arcz = ".1 12 23 34 45 53 3."
         self.check_coverage("""\
             o = [(1,2), (3,4)]
             o = [a for a in o]
@@ -428,19 +562,49 @@ class LoopArcTest(CoverageTest):
                 x = tup[0]
                 y = tup[1]
             """,
-            arcz=arcz,
+            arcz=".1 -22 2-2 12 23 34 45 53 3.",
         )
-        if env.PY3:
-            arcz = ".1 12 -22 2-2 23 34 42 2."
-        else:
-            arcz = ".1 12 23 34 42 2."
         self.check_coverage("""\
             o = [(1,2), (3,4)]
             for tup in [a for a in o]:
                 x = tup[0]
                 y = tup[1]
             """,
-            arcz=arcz,
+            arcz=".1 12 -22 2-2 23 34 42 2.",
+        )
+
+    # https://bugs.python.org/issue44672
+    @pytest.mark.skipif(env.PYVERSION < (3, 10), reason="<3.10 traced final pass incorrectly")
+    def test_incorrect_loop_exit_bug_1175(self):
+        self.check_coverage("""\
+            def wrong_loop(x):
+                if x:
+                    for i in [3, 33]:
+                        print(i+4)
+                else:
+                    pass
+
+            wrong_loop(8)
+            """,
+            arcz=".1 .2 23 26 34 43 3. 6. 18 8.",
+            arcz_missing="26 6.",
+        )
+
+    # https://bugs.python.org/issue44672
+    @pytest.mark.skipif(env.PYVERSION < (3, 10), reason="<3.10 traced final pass incorrectly")
+    def test_incorrect_if_bug_1175(self):
+        self.check_coverage("""\
+            def wrong_loop(x):
+                if x:
+                    if x:
+                        print(4)
+                else:
+                    pass
+
+            wrong_loop(8)
+            """,
+            arcz=".1 .2 23 26 34 4. 3. 6. 18 8.",
+            arcz_missing="26 3. 6.",
         )
 
     def test_generator_expression(self):
@@ -455,6 +619,21 @@ class LoopArcTest(CoverageTest):
             arcz=".1 -22 2-2 12 23 34 45 53 3.",
         )
 
+    def test_generator_expression_another_way(self):
+        # https://bugs.python.org/issue44450
+        # Generator expression:
+        self.check_coverage("""\
+            o = ((1,2), (3,4))
+            o = (a for
+                 a in
+                 o)
+            for tup in o:
+                x = tup[0]
+                y = tup[1]
+            """,
+            arcz=".1 -22 2-2 12 25 56 67 75 5.",
+        )
+
     def test_other_comprehensions(self):
         # Set comprehension:
         self.check_coverage("""\
@@ -694,6 +873,26 @@ class ExceptionArcTest(CoverageTest):
             arcz_missing="3D BC CD",
         )
 
+    def test_break_continue_without_finally(self):
+        self.check_coverage("""\
+            a, c, d, i = 1, 1, 1, 99
+            try:
+                for i in range(3):
+                    try:
+                        a = 5
+                        if i > 0:
+                            break
+                        continue
+                    except:
+                        c = 10
+            except:
+                d = 12                              # C
+            assert a == 5 and c == 1 and d == 1     # D
+            """,
+            arcz=".1 12 23 34 3D 45 56 67 68 7D 83 9A A3 BC CD D.",
+            arcz_missing="3D 9A A3 BC CD",
+        )
+
     def test_continue_through_finally(self):
         if env.PYBEHAVIOR.finally_jumps_back:
             arcz = ".1 12 23 34 3D 45 56 67 68 73 7A 8A A3 A7 BC CD D."
@@ -1086,11 +1285,9 @@ class YieldTest(CoverageTest):
                 ".2 23 34 45 52 2.",
             arcz_missing="2.",
         )
-        self.assertEqual(self.stdout(), "20\n12\n")
+        assert self.stdout() == "20\n12\n"
 
     def test_yield_from(self):
-        if not env.PYBEHAVIOR.yield_from:
-            self.skipTest("Python before 3.3 doesn't have 'yield from'")
         self.check_coverage("""\
             def gen(inp):
                 i = 2
@@ -1123,6 +1320,58 @@ class YieldTest(CoverageTest):
         )
 
 
+@pytest.mark.skipif(not env.PYBEHAVIOR.match_case, reason="Match-case is new in 3.10")
+@pytest.mark.skipif(env.PYVERSION <= (3, 10, 0, 'beta', 4, 0), reason="3.10.0b4 had bugs")
+class MatchCaseTest(CoverageTest):
+    """Tests of match-case."""
+    def test_match_case_with_default(self):
+        self.check_coverage("""\
+            for command in ["huh", "go home", "go n"]:
+                match command.split():
+                    case ["go", direction] if direction in "nesw":
+                        match = f"go: {direction}"
+                    case ["go", _]:
+                        match = "no go"
+                    case _:
+                        match = "default"
+                print(match)
+            """,
+            arcz=".1 12 23 34 49 35 56 69 57 78 89 91 1.",
+        )
+        assert self.stdout() == "default\nno go\ngo: n\n"
+
+    def test_match_case_with_wildcard(self):
+        self.check_coverage("""\
+            for command in ["huh", "go home", "go n"]:
+                match command.split():
+                    case ["go", direction] if direction in "nesw":
+                        match = f"go: {direction}"
+                    case ["go", _]:
+                        match = "no go"
+                    case x:
+                        match = f"default: {x}"
+                print(match)
+            """,
+            arcz=".1 12 23 34 49 35 56 69 57 78 89 91 1.",
+        )
+        assert self.stdout() == "default: ['huh']\nno go\ngo: n\n"
+
+    def test_match_case_without_wildcard(self):
+        self.check_coverage("""\
+            match = None
+            for command in ["huh", "go home", "go n"]:
+                match command.split():
+                    case ["go", direction] if direction in "nesw":
+                        match = f"go: {direction}"
+                    case ["go", _]:
+                        match = "no go"
+                print(match)
+            """,
+            arcz=".1 12 23 34 45 58 46 78 67 68 82 2.",
+        )
+        assert self.stdout() == "None\nno go\ngo: n\n"
+
+
 class OptimizedIfTest(CoverageTest):
     """Tests of if statements being optimized away."""
 
@@ -1336,8 +1585,6 @@ class MiscArcTest(CoverageTest):
         )
 
     def test_unpacked_literals(self):
-        if not env.PYBEHAVIOR.unpackings_pep448:
-            self.skipTest("Don't have unpacked literals until 3.5")
         self.check_coverage("""\
             d = {
                 'a': 2,
@@ -1374,7 +1621,8 @@ class MiscArcTest(CoverageTest):
         # opcodes.
         # Note that we no longer interpret bytecode at all, but it couldn't
         # hurt to keep the test...
-        for n in [10, 50, 100, 500, 1000, 2000]:
+        sizes = [10, 50, 100, 500, 1000, 2000]
+        for n in sizes:
             code = """\
                 data = [
                 """ + "".join("""\
@@ -1387,7 +1635,7 @@ class MiscArcTest(CoverageTest):
                 print(len(data))
                 """
             self.check_coverage(code, arcs=[(-1, 1), (1, 2*n+4), (2*n+4, -1)])
-            self.assertEqual(self.stdout().split()[-1], str(n))
+        assert self.stdout().split() == [str(n) for n in sizes]
 
     def test_partial_generators(self):
         # https://github.com/nedbat/coveragepy/issues/475
@@ -1406,18 +1654,10 @@ class MiscArcTest(CoverageTest):
             arcz=".1 17 7.  .2 23 34 45 5.  -22 2-2  -33 3-3  -44 4-4",
             arcz_missing="3-3 -44 4-4",
         )
-        # ugh, unexposed methods??
-        filename = self.last_module_name + ".py"
-        fr = cov._get_file_reporter(filename)
-        arcs_executed = cov._analyze(filename).arcs_executed()
-        self.assertEqual(
-            fr.missing_arc_description(3, -3, arcs_executed),
-            "line 3 didn't finish the generator expression on line 3"
-        )
-        self.assertEqual(
-            fr.missing_arc_description(4, -4, arcs_executed),
-            "line 4 didn't run the generator expression on line 4"
-        )
+        expected = "line 3 didn't finish the generator expression on line 3"
+        assert self.get_missing_arc_description(cov, 3, -3) == expected
+        expected = "line 4 didn't run the generator expression on line 4"
+        assert self.get_missing_arc_description(cov, 4, -4) == expected
 
 
 class DecoratorArcTest(CoverageTest):
@@ -1590,11 +1830,6 @@ class LambdaArcTest(CoverageTest):
 class AsyncTest(CoverageTest):
     """Tests of the new async and await keywords in Python 3.5"""
 
-    def setUp(self):
-        if not env.PYBEHAVIOR.async_syntax:
-            self.skipTest("Async features are new in Python 3.5")
-        super(AsyncTest, self).setUp()
-
     def test_async(self):
         self.check_coverage("""\
             import asyncio
@@ -1620,7 +1855,7 @@ class AsyncTest(CoverageTest):
                 "-89 9C C-8",
             arcz_unpredicted="5-3 9-8",
         )
-        self.assertEqual(self.stdout(), "Compute 1 + 2 ...\n1 + 2 = 3\n")
+        assert self.stdout() == "Compute 1 + 2 ...\n1 + 2 = 3\n"
 
     def test_async_for(self):
         self.check_coverage("""\
@@ -1657,16 +1892,22 @@ class AsyncTest(CoverageTest):
                 "-AB BC C-A DE E-A ",       # __anext__
             arcz_unpredicted="CD",
         )
-        self.assertEqual(self.stdout(), "a\nb\nc\n.\n")
+        assert self.stdout() == "a\nb\nc\n.\n"
 
     def test_async_with(self):
+        if env.PYBEHAVIOR.exit_through_with:
+            arcz = ".1 1. .2 23 32 2."
+            arcz_missing = ".2 23 32 2."
+        else:
+            arcz = ".1 1. .2 23 3."
+            arcz_missing = ".2 23 3."
         self.check_coverage("""\
             async def go():
                 async with x:
                     pass
             """,
-            arcz=".1 1. .2 23 3.",
-            arcz_missing=".2 23 3.",
+            arcz=arcz,
+            arcz_missing=arcz_missing,
         )
 
     def test_async_decorator(self):
@@ -1686,6 +1927,97 @@ class AsyncTest(CoverageTest):
             arcz_missing='-46 6-4',
         )
 
+    # https://github.com/nedbat/coveragepy/issues/1158
+    # https://bugs.python.org/issue44621
+    @pytest.mark.skipif(env.PYVERSION[:2] == (3, 9), reason="avoid a 3.9 bug: 44621")
+    @pytest.mark.skipif(env.PYVERSION < (3, 7), reason="need asyncio.run")
+    def test_bug1158(self):
+        self.check_coverage("""\
+            import asyncio
+
+            async def async_gen():
+                yield 4
+
+            async def async_test():
+                global a
+                a = 8
+                async for i in async_gen():
+                    print(i + 10)
+                else:
+                    a = 12
+
+            asyncio.run(async_test())
+            assert a == 12
+            """,
+            arcz=".1 13 36 6E EF F.  -34 4-3  -68 89 9A 9C A9 C-6",
+        )
+        assert self.stdout() == "14\n"
+
+    # https://github.com/nedbat/coveragepy/issues/1176
+    # https://bugs.python.org/issue44622
+    @pytest.mark.skipif(
+        (3, 10, 0, "alpha", 0, 0) <= env.PYVERSION[:6] <= (3, 10, 0, "beta", 4, 0),
+        reason="avoid a 3.10 bug fixed after beta 4: 44622"
+    )
+    @pytest.mark.skipif(env.PYVERSION < (3, 7), reason="need asyncio.run")
+    def test_bug1176(self):
+        self.check_coverage("""\
+            import asyncio
+
+            async def async_gen():
+                yield 4
+
+            async def async_test():
+                async for i in async_gen():
+                    print(i + 8)
+
+            asyncio.run(async_test())
+            """,
+            arcz=".1 13 36 6A A.  -34 4-3  -67 78 87 7-6",
+        )
+        assert self.stdout() == "12\n"
+
+    # https://github.com/nedbat/coveragepy/issues/1205
+    # https://bugs.python.org/issue44840
+    @pytest.mark.skipif(
+        (3, 10, 0, "alpha", 0, 0) <= env.PYVERSION[:6] <= (3, 10, 0, "candidate", 1, 0),
+        reason="avoid a 3.10 bug fixed after rc1: 44840"
+    )
+    def test_bug1205(self):
+        self.check_coverage("""\
+            def func():
+                if T(2):
+                    if T(3):
+                        if F(4):
+                            if X(5):
+                                return 6
+                    else:
+                        return 8
+                elif X(9) and Y:
+                    return 10
+
+            T, F = (lambda _: True), (lambda _: False)
+            func()
+            """,
+            arcz=".1 1C CD D.  .2 23 29 34 38 45 4. 56 5. 6. 8. 9. 9A A.  -CC C-C",
+            arcz_missing="29 38 45 56 5. 6. 8. 9. 9A A.",
+        )
+
+
+class AnnotationTest(CoverageTest):
+    """Tests using type annotations."""
+
+    def test_annotations(self):
+        self.check_coverage("""\
+            def f(x:str, y:int) -> str:
+                a:int = 2
+                return f"{x}, {y}, {a}, 3"
+            print(f("x", 4))
+            """,
+            arcz=".1 .2 23 3.  14 4.",
+        )
+        assert self.stdout() == "x, 4, 2, 3\n"
+
 
 class ExcludeTest(CoverageTest):
     """Tests of exclusions to indicate known partial branches."""
@@ -1739,4 +2071,4 @@ class LineDataTest(CoverageTest):
 
         data = cov.get_data()
         fun1_lines = data.lines(abs_file("fun1.py"))
-        self.assertCountEqual(fun1_lines, [1, 2, 5])
+        assert_count_equal(fun1_lines, [1, 2, 5])
