@@ -6,17 +6,19 @@
 import glob
 import os
 import random
+import re
 import sys
 import threading
 import time
 
 from flaky import flaky
+import pytest
 
 import coverage
 from coverage import env
-from coverage.backward import import_local_file
 from coverage.data import line_counts
 from coverage.files import abs_file
+from coverage.misc import import_local_file
 
 from tests.coveragetest import CoverageTest
 from tests.helpers import remove_files
@@ -91,7 +93,7 @@ class LineCountTest(CoverageTest):
             print("done")
             """
 
-        self.assertEqual(line_count(CODE), 5)
+        assert line_count(CODE) == 5
 
 
 # The code common to all the concurrency models.
@@ -144,13 +146,7 @@ PRINT_SUM_RANGE = """
     """
 
 # Import the things to use threads.
-if env.PY2:
-    THREAD = """
-    import threading
-    import Queue as queue
-    """
-else:
-    THREAD = """
+THREAD = """
     import threading
     import queue
     """
@@ -227,14 +223,14 @@ class ConcurrencyTest(CoverageTest):
         expected_cant_trace = cant_trace_msg(concurrency, the_module)
 
         if expected_cant_trace is not None:
-            self.assertEqual(out, expected_cant_trace)
+            assert out == expected_cant_trace
         else:
             # We can fully measure the code if we are using the C tracer, which
             # can support all the concurrency, or if we are using threads.
             if expected_out is None:
                 expected_out = "%d\n" % (sum(range(self.QLIMIT)))
             print(code)
-            self.assertEqual(out, expected_out)
+            assert out == expected_out
 
             # Read the coverage file and see that try_it.py has all its lines
             # executed.
@@ -244,11 +240,11 @@ class ConcurrencyTest(CoverageTest):
             # If the test fails, it's helpful to see this info:
             fname = abs_file("try_it.py")
             linenos = data.lines(fname)
-            print("{}: {}".format(len(linenos), linenos))
+            print(f"{len(linenos)}: {linenos}")
             print_simple_annotation(code, linenos)
 
             lines = line_count(code)
-            self.assertEqual(line_counts(data)['try_it.py'], lines)
+            assert line_counts(data)['try_it.py'] == lines
 
     def test_threads(self):
         code = (THREAD + SUM_RANGE_Q + PRINT_SUM_RANGE).format(QLIMIT=self.QLIMIT)
@@ -340,8 +336,8 @@ MULTI_CODE = """
     def process_worker_main(args):
         # Need to pause, or the tasks go too quickly, and some processes
         # in the pool don't get any work, and then don't record data.
-        time.sleep(0.02)
         ret = work(*args)
+        time.sleep(0.1)
         return os.getpid(), ret
 
     if __name__ == "__main__":      # pragma: no branch
@@ -362,15 +358,11 @@ MULTI_CODE = """
     """
 
 
+@pytest.mark.skipif(not multiprocessing, reason="No multiprocessing in this Python")
 @flaky(max_runs=30)         # Sometimes a test fails due to inherent randomness. Try more times.
 class MultiprocessingTest(CoverageTest):
     """Test support of the multiprocessing module."""
 
-    def setUp(self):
-        if not multiprocessing:
-            self.skipTest("No multiprocessing in this Python")      # pragma: only jython
-        super(MultiprocessingTest, self).setUp()
-
     def try_multiprocessing_code(
         self, code, expected_out, the_module, nprocs, concurrency="multiprocessing", args=""
     ):
@@ -382,12 +374,7 @@ class MultiprocessingTest(CoverageTest):
             source = .
             """ % concurrency)
 
-        if env.PYVERSION >= (3, 4):
-            start_methods = ['fork', 'spawn']
-        else:
-            start_methods = ['']
-
-        for start_method in start_methods:
+        for start_method in ["fork", "spawn"]:
             if start_method and start_method not in multiprocessing.get_all_start_methods():
                 continue
 
@@ -399,24 +386,29 @@ class MultiprocessingTest(CoverageTest):
             expected_cant_trace = cant_trace_msg(concurrency, the_module)
 
             if expected_cant_trace is not None:
-                self.assertEqual(out, expected_cant_trace)
+                assert out == expected_cant_trace
             else:
-                self.assertEqual(out.rstrip(), expected_out)
-                self.assertEqual(len(glob.glob(".coverage.*")), nprocs + 1)
+                assert out.rstrip() == expected_out
+                assert len(glob.glob(".coverage.*")) == nprocs + 1
 
                 out = self.run_command("coverage combine")
-                self.assertEqual(out, "")
+                out_lines = out.splitlines()
+                assert len(out_lines) == nprocs + 1
+                assert all(
+                    re.fullmatch(r"Combined data file \.coverage\..*\.\d+\.\d+", line)
+                    for line in out_lines
+                )
                 out = self.run_command("coverage report -m")
 
                 last_line = self.squeezed_lines(out)[-1]
-                self.assertRegex(last_line, r"TOTAL \d+ 0 100%")
+                assert re.search(r"TOTAL \d+ 0 100%", last_line)
 
     def test_multiprocessing_simple(self):
         nprocs = 3
         upto = 30
         code = (SQUARE_OR_CUBE_WORK + MULTI_CODE).format(NPROCS=nprocs, UPTO=upto)
         total = sum(x*x if x%2 else x*x*x for x in range(upto))
-        expected_out = "{nprocs} pids, total = {total}".format(nprocs=nprocs, total=total)
+        expected_out = f"{nprocs} pids, total = {total}"
         self.try_multiprocessing_code(code, expected_out, threading, nprocs)
 
     def test_multiprocessing_append(self):
@@ -424,7 +416,7 @@ class MultiprocessingTest(CoverageTest):
         upto = 30
         code = (SQUARE_OR_CUBE_WORK + MULTI_CODE).format(NPROCS=nprocs, UPTO=upto)
         total = sum(x*x if x%2 else x*x*x for x in range(upto))
-        expected_out = "{nprocs} pids, total = {total}".format(nprocs=nprocs, total=total)
+        expected_out = f"{nprocs} pids, total = {total}"
         self.try_multiprocessing_code(code, expected_out, threading, nprocs, args="--append")
 
     def test_multiprocessing_and_gevent(self):
@@ -434,13 +426,17 @@ class MultiprocessingTest(CoverageTest):
             SUM_RANGE_WORK + EVENTLET + SUM_RANGE_Q + MULTI_CODE
         ).format(NPROCS=nprocs, UPTO=upto)
         total = sum(sum(range((x + 1) * 100)) for x in range(upto))
-        expected_out = "{nprocs} pids, total = {total}".format(nprocs=nprocs, total=total)
+        expected_out = f"{nprocs} pids, total = {total}"
         self.try_multiprocessing_code(
             code, expected_out, eventlet, nprocs, concurrency="multiprocessing,eventlet"
         )
 
-    def try_multiprocessing_code_with_branching(self, code, expected_out):
-        """Run code using multiprocessing, it should produce `expected_out`."""
+    def test_multiprocessing_with_branching(self):
+        nprocs = 3
+        upto = 30
+        code = (SQUARE_OR_CUBE_WORK + MULTI_CODE).format(NPROCS=nprocs, UPTO=upto)
+        total = sum(x*x if x%2 else x*x*x for x in range(upto))
+        expected_out = f"{nprocs} pids, total = {total}"
         self.make_file("multi.py", code)
         self.make_file("multi.rc", """\
             [run]
@@ -449,32 +445,24 @@ class MultiprocessingTest(CoverageTest):
             omit = */site-packages/*
             """)
 
-        if env.PYVERSION >= (3, 4):
-            start_methods = ['fork', 'spawn']
-        else:
-            start_methods = ['']
-
-        for start_method in start_methods:
+        for start_method in ["fork", "spawn"]:
             if start_method and start_method not in multiprocessing.get_all_start_methods():
                 continue
 
-            out = self.run_command("coverage run --rcfile=multi.rc multi.py %s" % (start_method,))
-            self.assertEqual(out.rstrip(), expected_out)
+            out = self.run_command(f"coverage run --rcfile=multi.rc multi.py {start_method}")
+            assert out.rstrip() == expected_out
 
             out = self.run_command("coverage combine")
-            self.assertEqual(out, "")
+            out_lines = out.splitlines()
+            assert len(out_lines) == nprocs + 1
+            assert all(
+                re.fullmatch(r"Combined data file \.coverage\..*\.\d+\.\d+", line)
+                for line in out_lines
+            )
             out = self.run_command("coverage report -m")
 
             last_line = self.squeezed_lines(out)[-1]
-            self.assertRegex(last_line, r"TOTAL \d+ 0 \d+ 0 100%")
-
-    def test_multiprocessing_with_branching(self):
-        nprocs = 3
-        upto = 30
-        code = (SQUARE_OR_CUBE_WORK + MULTI_CODE).format(NPROCS=nprocs, UPTO=upto)
-        total = sum(x*x if x%2 else x*x*x for x in range(upto))
-        expected_out = "{nprocs} pids, total = {total}".format(nprocs=nprocs, total=total)
-        self.try_multiprocessing_code_with_branching(code, expected_out)
+            assert re.search(r"TOTAL \d+ 0 \d+ 0 100%", last_line)
 
     def test_multiprocessing_bootstrap_error_handling(self):
         # An exception during bootstrapping will be reported.
@@ -490,8 +478,8 @@ class MultiprocessingTest(CoverageTest):
             _crash = _bootstrap
             """)
         out = self.run_command("coverage run multi.py")
-        self.assertIn("Exception during multiprocessing bootstrap init", out)
-        self.assertIn("Exception: Crashing because called by _bootstrap", out)
+        assert "Exception during multiprocessing bootstrap init" in out
+        assert "Exception: Crashing because called by _bootstrap" in out
 
     def test_bug890(self):
         # chdir in multiprocessing shouldn't keep us from finding the
@@ -510,7 +498,7 @@ class MultiprocessingTest(CoverageTest):
             concurrency = multiprocessing
             """)
         out = self.run_command("coverage run multi.py")
-        self.assertEqual(out.splitlines()[-1], "ok")
+        assert out.splitlines()[-1] == "ok"
 
 
 def test_coverage_stop_in_threads():
@@ -520,7 +508,7 @@ def test_coverage_stop_in_threads():
     def run_thread():           # pragma: nested
         """Check that coverage is stopping properly in threads."""
         deadline = time.time() + 5
-        ident = threading.currentThread().ident
+        ident = threading.current_thread().ident
         if sys.gettrace() is not None:
             has_started_coverage.append(ident)
         while sys.gettrace() is not None:
@@ -549,7 +537,7 @@ def test_thread_safe_save_data(tmpdir):
 
     # Create some Python modules and put them in the path
     modules_dir = tmpdir.mkdir('test_modules')
-    module_names = ["m{:03d}".format(i) for i in range(1000)]
+    module_names = [f"m{i:03d}" for i in range(1000)]
     for module_name in module_names:
         modules_dir.join(module_name + ".py").write("def f(): pass\n")
 
