@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Licensed under the Apache License: http://www.apache.org/licenses/LICENSE-2.0
 # For details: https://github.com/nedbat/coveragepy/blob/master/NOTICE.txt
 
@@ -12,21 +11,20 @@ import os.path
 import re
 import sys
 
-import mock
+from unittest import mock
 import pytest
-from unittest_mixins import change_dir
 
 import coverage
-from coverage.backward import unicode_class
 from coverage import env
+from coverage.exceptions import CoverageException, NotPython, NoSource
 from coverage.files import abs_file, flat_rootname
 import coverage.html
-from coverage.misc import CoverageException, NotPython, NoSource
 from coverage.report import get_analysis_to_report
 
 from tests.coveragetest import CoverageTest, TESTS_DIR
 from tests.goldtest import gold_path
 from tests.goldtest import compare, contains, doesnt_contain, contains_any
+from tests.helpers import assert_coverage_warnings, change_dir
 
 
 class HtmlTestHelpers(CoverageTest):
@@ -58,7 +56,7 @@ class HtmlTestHelpers(CoverageTest):
 
     def get_html_report_content(self, module):
         """Return the content of the HTML report for `module`."""
-        filename = module.replace(".", "_").replace("/", "_") + ".html"
+        filename = flat_rootname(module) + ".html"
         filename = os.path.join("htmlcov", filename)
         with open(filename) as f:
             return f.read()
@@ -94,11 +92,11 @@ class HtmlTestHelpers(CoverageTest):
         self.assert_recent_datetime(
             timestamp,
             seconds=120,
-            msg="Timestamp is wrong: {}".format(timestamp),
+            msg=f"Timestamp is wrong: {timestamp}",
         )
 
 
-class FileWriteTracker(object):
+class FileWriteTracker:
     """A fake object to track how `open` is used to write files."""
     def __init__(self, written):
         self.written = written
@@ -113,8 +111,8 @@ class FileWriteTracker(object):
 class HtmlDeltaTest(HtmlTestHelpers, CoverageTest):
     """Tests of the HTML delta speed-ups."""
 
-    def setUp(self):
-        super(HtmlDeltaTest, self).setUp()
+    def setup_test(self):
+        super().setup_test()
 
         # At least one of our tests monkey-patches the version of coverage.py,
         # so grab it here to restore it later.
@@ -136,7 +134,7 @@ class HtmlDeltaTest(HtmlTestHelpers, CoverageTest):
         self.files_written = set()
         mock_open = FileWriteTracker(self.files_written).open
         with mock.patch("coverage.html.open", mock_open):
-            return super(HtmlDeltaTest, self).run_coverage(covargs=covargs, htmlargs=htmlargs)
+            return super().run_coverage(covargs=covargs, htmlargs=htmlargs)
 
     def assert_htmlcov_files_exist(self):
         """Assert that all the expected htmlcov files exist."""
@@ -343,13 +341,14 @@ class HtmlWithUnparsableFilesTest(HtmlTestHelpers, CoverageTest):
         self.make_file("innocuous.py", "a = 2")
         cov = coverage.Coverage()
         self.start_import_stop(cov, "main")
+
         self.make_file("innocuous.py", "<h1>This isn't python!</h1>")
-        cov.html_report(ignore_errors=True)
-        msg = "Expected a warning to be thrown when an invalid python file is parsed"
-        assert 1 == len(cov._warnings), msg
-        msg = "Warning message should be in 'invalid file' warning"
-        assert "Couldn't parse Python file" in cov._warnings[0], msg
-        assert "innocuous.py" in cov._warnings[0], "Filename should be in 'invalid file' warning"
+        with pytest.warns(Warning) as warns:
+            cov.html_report(ignore_errors=True)
+        assert_coverage_warnings(
+            warns,
+            re.compile(r"Couldn't parse Python file '.*innocuous.py' \(couldnt-parse\)"),
+            )
         self.assert_exists("htmlcov/index.html")
         # This would be better as a glob, if the HTML layout changes:
         self.assert_doesnt_exist("htmlcov/innocuous.html")
@@ -507,6 +506,8 @@ class HtmlTest(HtmlTestHelpers, CoverageTest):
         self.assert_exists("htmlcov/index.html")
         self.assert_doesnt_exist("htmlcov/main_file_py.html")
         self.assert_exists("htmlcov/not_covered_py.html")
+        index = self.get_html_index_content()
+        assert "1 file skipped due to complete coverage." in index
 
     def test_report_skip_covered_branches(self):
         self.make_main_and_not_covered()
@@ -542,6 +543,8 @@ class HtmlTest(HtmlTestHelpers, CoverageTest):
         self.assert_exists("htmlcov/index.html")
         self.assert_exists("htmlcov/main_file_py.html")
         self.assert_doesnt_exist("htmlcov/submodule___init___py.html")
+        index = self.get_html_index_content()
+        assert "1 empty file skipped." in index
 
     def test_html_skip_empty(self):
         self.make_init_and_main()
@@ -555,8 +558,8 @@ class HtmlTest(HtmlTestHelpers, CoverageTest):
 class HtmlStaticFileTest(CoverageTest):
     """Tests of the static file copying for the HTML report."""
 
-    def setUp(self):
-        super(HtmlStaticFileTest, self).setUp()
+    def setup_test(self):
+        super().setup_test()
         original_path = list(coverage.html.STATIC_PATH)
         self.addCleanup(setattr, coverage.html, 'STATIC_PATH', original_path)
 
@@ -618,7 +621,7 @@ def filepath_to_regex(path):
     return regex
 
 
-def compare_html(expected, actual):
+def compare_html(expected, actual, extra_scrubs=None):
     """Specialized compare function for our HTML files."""
     scrubs = [
         (r'/coverage.readthedocs.io/?[-.\w/]*', '/coverage.readthedocs.io/VER'),
@@ -629,22 +632,25 @@ def compare_html(expected, actual):
         (r'<span class="(nam|key)">(print|True|False)</span>', r'<span class="nam">\2</span>'),
         # Occasionally an absolute path is in the HTML report.
         (filepath_to_regex(TESTS_DIR), 'TESTS_DIR'),
-        (filepath_to_regex(flat_rootname(unicode_class(TESTS_DIR))), '_TESTS_DIR'),
+        (filepath_to_regex(flat_rootname(str(TESTS_DIR))), '_TESTS_DIR'),
         # The temp dir the tests make.
         (filepath_to_regex(os.getcwd()), 'TEST_TMPDIR'),
-        (filepath_to_regex(flat_rootname(unicode_class(os.getcwd()))), '_TEST_TMPDIR'),
+        (filepath_to_regex(flat_rootname(str(os.getcwd()))), '_TEST_TMPDIR'),
         (filepath_to_regex(abs_file(os.getcwd())), 'TEST_TMPDIR'),
-        (filepath_to_regex(flat_rootname(unicode_class(abs_file(os.getcwd())))), '_TEST_TMPDIR'),
+        (filepath_to_regex(flat_rootname(str(abs_file(os.getcwd())))), '_TEST_TMPDIR'),
+        # Old format of test directories that could be in the gold files.
         (r'/private/var/folders/[\w/]{35}/coverage_test/tests_test_html_\w+_\d{8}', 'TEST_TMPDIR'),
         (r'_private_var_folders_\w{35}_coverage_test_tests_test_html_\w+_\d{8}', '_TEST_TMPDIR'),
     ]
     if env.WINDOWS:
         # For file paths...
         scrubs += [(r"\\", "/")]
+    if extra_scrubs:
+        scrubs += extra_scrubs
     compare(expected, actual, file_pattern="*.html", scrubs=scrubs)
 
 
-class HtmlGoldTests(CoverageTest):
+class HtmlGoldTest(CoverageTest):
     """Tests of HTML reporting that use gold files."""
 
     def test_a(self):
@@ -898,7 +904,12 @@ assert len(math) == 18
         for p in glob.glob("out/other/*_other_py.html"):
             os.rename(p, "out/other/blah_blah_other_py.html")
 
-        compare_html(gold_path("html/other"), "out/other")
+        compare_html(
+            gold_path("html/other"), "out/other",
+            extra_scrubs=[
+                (r'href="d_[0-9a-z]{16}_', 'href="_TEST_TMPDIR_othersrc_'),
+                ],
+            )
         contains(
             "out/other/index.html",
             '<a href="here_py.html">here.py</a>',
@@ -1040,9 +1051,7 @@ assert len(math) == 18
         doesnt_contain("out/tabbed_py.html", "\t")
 
     def test_unicode(self):
-        surrogate = u"\U000e0100"
-        if env.PY2:
-            surrogate = surrogate.encode('utf-8')
+        surrogate = "\U000e0100"
 
         self.make_file("unicode.py", """\
             # -*- coding: utf-8 -*-
