@@ -11,8 +11,8 @@ import pytest
 
 import coverage
 from coverage import env
-from coverage.backward import import_local_file
 from coverage.files import abs_file
+from coverage.misc import import_local_file
 
 from tests.coveragetest import CoverageTest
 from tests import osinfo
@@ -80,18 +80,19 @@ class RecursionTest(CoverageTest):
 
     def test_long_recursion(self):
         # We can't finish a very deep recursion, but we don't crash.
-        with self.assertRaises(RuntimeError):
-            self.check_coverage("""\
-                def recur(n):
-                    if n == 0:
-                        return 0
-                    else:
-                        return recur(n-1)+1
+        with pytest.raises(RuntimeError):
+            with pytest.warns(None):
+                self.check_coverage("""\
+                    def recur(n):
+                        if n == 0:
+                            return 0
+                        else:
+                            return recur(n-1)+1
 
-                recur(100000)  # This is definitely too many frames.
-                """,
-                [1, 2, 3, 5, 7], ""
-                )
+                    recur(100000)  # This is definitely too many frames.
+                    """,
+                    [1, 2, 3, 5, 7], ""
+                    )
 
     def test_long_recursion_recovery(self):
         # Test the core of bug 93: https://github.com/nedbat/coveragepy/issues/93
@@ -117,7 +118,8 @@ class RecursionTest(CoverageTest):
             """)
 
         cov = coverage.Coverage()
-        self.start_import_stop(cov, "recur")
+        with pytest.warns(None):
+            self.start_import_stop(cov, "recur")
 
         pytrace = (cov._collector.tracer_name() == "PyTracer")
         expected_missing = [3]
@@ -125,16 +127,14 @@ class RecursionTest(CoverageTest):
             expected_missing += [9, 10, 11]
 
         _, statements, missing, _ = cov.analysis("recur.py")
-        self.assertEqual(statements, [1, 2, 3, 5, 7, 8, 9, 10, 11])
-        self.assertEqual(expected_missing, missing)
+        assert statements == [1, 2, 3, 5, 7, 8, 9, 10, 11]
+        assert expected_missing == missing
 
         # Get a warning about the stackoverflow effect on the tracing function.
         if pytrace:                                 # pragma: no metacov
-            self.assertEqual(cov._warnings,
-                ["Trace function changed, measurement is likely wrong: None"]
-                )
+            assert cov._warnings == ["Trace function changed, measurement is likely wrong: None"]
         else:
-            self.assertEqual(cov._warnings, [])
+            assert cov._warnings == []
 
 
 class MemoryLeakTest(CoverageTest):
@@ -147,10 +147,8 @@ class MemoryLeakTest(CoverageTest):
 
     """
     @flaky
+    @pytest.mark.skipif(env.JYTHON, reason="Don't bother on Jython")
     def test_for_leaks(self):
-        if env.JYTHON:
-            self.skipTest("Don't bother on Jython")
-
         # Our original bad memory leak only happened on line numbers > 255, so
         # make a code object with more lines than that.  Ugly string mumbo
         # jumbo to get 300 blank lines at the beginning..
@@ -190,17 +188,16 @@ class MemoryLeakTest(CoverageTest):
                 fails += 1                                  # pragma: only failure
 
         if fails > 8:
-            self.fail("RAM grew by %d" % (ram_growth))      # pragma: only failure
+            pytest.fail("RAM grew by %d" % (ram_growth))      # pragma: only failure
 
 
 class MemoryFumblingTest(CoverageTest):
     """Test that we properly manage the None refcount."""
 
+    @pytest.mark.skipif(not env.C_TRACER, reason="Only the C tracer has refcounting issues")
     def test_dropping_none(self):                           # pragma: not covered
-        if not env.C_TRACER:
-            self.skipTest("Only the C tracer has refcounting issues")
         # TODO: Mark this so it will only be run sometimes.
-        self.skipTest("This is too expensive for now (30s)")
+        pytest.skip("This is too expensive for now (30s)")
         # Start and stop coverage thousands of times to flush out bad
         # reference counting, maybe.
         self.make_file("the_code.py", """\
@@ -224,18 +221,16 @@ class MemoryFumblingTest(CoverageTest):
             print("Final None refcount: %d" % (sys.getrefcount(None)))
             """)
         status, out = self.run_command_status("python main.py")
-        self.assertEqual(status, 0)
-        self.assertIn("Final None refcount", out)
-        self.assertNotIn("Fatal", out)
+        assert status == 0
+        assert "Final None refcount" in out
+        assert "Fatal" not in out
 
 
+@pytest.mark.skipif(env.JYTHON, reason="Pyexpat isn't a problem on Jython")
 class PyexpatTest(CoverageTest):
     """Pyexpat screws up tracing. Make sure we've counter-defended properly."""
 
     def test_pyexpat(self):
-        if env.JYTHON:
-            self.skipTest("Pyexpat isn't a problem on Jython")
-
         # pyexpat calls the trace function explicitly (inexplicably), and does
         # it wrong for exceptions.  Parsing a DOCTYPE for some reason throws
         # an exception internally, and triggers its wrong behavior.  This test
@@ -268,20 +263,18 @@ class PyexpatTest(CoverageTest):
         self.start_import_stop(cov, "outer")
 
         _, statements, missing, _ = cov.analysis("trydom.py")
-        self.assertEqual(statements, [1, 3, 8, 9, 10, 11, 13])
-        self.assertEqual(missing, [])
+        assert statements == [1, 3, 8, 9, 10, 11, 13]
+        assert missing == []
 
         _, statements, missing, _ = cov.analysis("outer.py")
-        self.assertEqual(statements, [101, 102])
-        self.assertEqual(missing, [])
+        assert statements == [101, 102]
+        assert missing == []
 
         # Make sure pyexpat isn't recorded as a source file.
         # https://github.com/nedbat/coveragepy/issues/419
         files = cov.get_data().measured_files()
-        self.assertFalse(
-            any(f.endswith("pyexpat.c") for f in files),
-            "Pyexpat.c is in the measured files!: %r:" % (files,)
-        )
+        msg = f"Pyexpat.c is in the measured files!: {files!r}:"
+        assert not any(f.endswith("pyexpat.c") for f in files), msg
 
 
 class ExceptionTest(CoverageTest):
@@ -393,7 +386,7 @@ class ExceptionTest(CoverageTest):
                 for lines in lines_expected.values():
                     lines[:] = [l for l in lines if l not in invisible]
 
-            self.assertEqual(clean_lines, lines_expected)
+            assert clean_lines == lines_expected
 
 
 class DoctestTest(CoverageTest):
@@ -451,19 +444,21 @@ class GettraceTest(CoverageTest):
         cov = coverage.Coverage(source=["sample"])
         self.start_import_stop(cov, "main")
 
-        self.assertEqual(self.stdout(), "10\n7\n3\n5\n9\n12\n")
+        assert self.stdout() == "10\n7\n3\n5\n9\n12\n"
 
         _, statements, missing, _ = cov.analysis("sample.py")
-        self.assertEqual(statements, [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12])
-        self.assertEqual(missing, [])
+        assert statements == [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12]
+        assert missing == []
 
     def test_setting_new_trace_function(self):
         # https://github.com/nedbat/coveragepy/issues/436
         self.check_coverage('''\
+            import os.path
             import sys
 
             def tracer(frame, event, arg):
-                print("%s: %s @ %d" % (event, frame.f_code.co_filename, frame.f_lineno))
+                filename = os.path.basename(frame.f_code.co_filename)
+                print("%s: %s @ %d" % (event, filename, frame.f_lineno))
                 return tracer
 
             def begin():
@@ -483,28 +478,24 @@ class GettraceTest(CoverageTest):
             a = 21
             b = 22
             ''',
-            lines=[1, 3, 4, 5, 7, 8, 10, 11, 12, 14, 15, 16, 18, 19, 20, 21, 22],
-            missing="4-5, 11-12",
+            lines=[1, 2, 4, 5, 6, 7, 9, 10, 12, 13, 14, 16, 17, 18, 20, 21, 22, 23, 24],
+            missing="5-7, 13-14",
         )
 
         out = self.stdout().replace(self.last_module_name, "coverage_test")
-
-        self.assertEqual(
-            out,
-            (
-                "call: coverage_test.py @ 10\n"
-                "line: coverage_test.py @ 11\n"
-                "line: coverage_test.py @ 12\n"
-                "return: coverage_test.py @ 12\n"
-            ),
+        expected = (
+            "call: coverage_test.py @ 12\n"
+            "line: coverage_test.py @ 13\n"
+            "line: coverage_test.py @ 14\n"
+            "return: coverage_test.py @ 14\n"
         )
+        assert expected == out
 
     @pytest.mark.expensive
-    def test_atexit_gettrace(self):             # pragma: no metacov
+    @pytest.mark.skipif(env.METACOV, reason="Can't set trace functions during meta-coverage")
+    def test_atexit_gettrace(self):
         # This is not a test of coverage at all, but of our understanding
         # of this edge-case behavior in various Pythons.
-        if env.METACOV:
-            self.skipTest("Can't set trace functions during meta-coverage")
 
         self.make_file("atexit_gettrace.py", """\
             import atexit, sys
@@ -523,13 +514,13 @@ class GettraceTest(CoverageTest):
             # This will show what the trace function is at the end of the program.
             """)
         status, out = self.run_command_status("python atexit_gettrace.py")
-        self.assertEqual(status, 0)
+        assert status == 0
         if env.PYPY and env.PYPYVERSION >= (5, 4):
             # Newer PyPy clears the trace function before atexit runs.
-            self.assertEqual(out, "None\n")
+            assert out == "None\n"
         else:
             # Other Pythons leave the trace function in place.
-            self.assertEqual(out, "trace_function\n")
+            assert out == "trace_function\n"
 
 
 class ExecTest(CoverageTest):
@@ -557,16 +548,14 @@ class ExecTest(CoverageTest):
         self.start_import_stop(cov, "main")
 
         _, statements, missing, _ = cov.analysis("main.py")
-        self.assertEqual(statements, [1, 2, 3, 4, 35])
-        self.assertEqual(missing, [])
+        assert statements == [1, 2, 3, 4, 35]
+        assert missing == []
         _, statements, missing, _ = cov.analysis("to_exec.py")
-        self.assertEqual(statements, [31])
-        self.assertEqual(missing, [])
+        assert statements == [31]
+        assert missing == []
 
     def test_unencodable_filename(self):
         # https://github.com/nedbat/coveragepy/issues/891
-        if env.PYVERSION < (3, 0):
-            self.skipTest("Python 2 can't seem to compile the file.")
         self.make_file("bug891.py", r"""exec(compile("pass", "\udcff.py", "exec"))""")
         cov = coverage.Coverage()
         self.start_import_stop(cov, "bug891")
@@ -588,8 +577,7 @@ class MockingProtectionTest(CoverageTest):
         # StopIteration error.
         self.make_file("bug416.py", """\
             import os.path
-
-            import mock
+            from unittest import mock
 
             @mock.patch('os.path.exists')
             def test_path_exists(mock_exists):
@@ -609,4 +597,4 @@ class MockingProtectionTest(CoverageTest):
         import py_compile
         py_compile.compile("bug416a.py")
         out = self.run_command("coverage run bug416.py")
-        self.assertEqual(out, "in test\nbug416a.py\n23\n17\n")
+        assert out == "in test\nbug416a.py\n23\n17\n"
