@@ -5,11 +5,13 @@
 
 import textwrap
 
+import pytest
+
 from coverage import env
-from coverage.misc import NotPython
+from coverage.exceptions import NotPython
 from coverage.parser import PythonParser
 
-from tests.coveragetest import CoverageTest, xfail
+from tests.coveragetest import CoverageTest
 from tests.helpers import arcz_to_arcs
 
 
@@ -20,8 +22,6 @@ class PythonParserTest(CoverageTest):
 
     def parse_source(self, text):
         """Parse `text` as source, and return the `PythonParser` used."""
-        if env.PY2:
-            text = text.decode("ascii")
         text = textwrap.dedent(text)
         parser = PythonParser(text=text, exclude="nocover")
         parser.parse_source()
@@ -40,9 +40,9 @@ class PythonParserTest(CoverageTest):
             class Bar:
                 pass
             """)
-        self.assertEqual(parser.exit_counts(), {
+        assert parser.exit_counts() == {
             2:1, 3:1, 4:2, 5:1, 7:1, 9:1, 10:1
-            })
+            }
 
     def test_generator_exit_counts(self):
         # https://github.com/nedbat/coveragepy/issues/324
@@ -53,12 +53,12 @@ class PythonParserTest(CoverageTest):
 
             list(gen([1,2,3]))
             """)
-        self.assertEqual(parser.exit_counts(), {
+        assert parser.exit_counts() == {
             1:1,    # def -> list
             2:2,    # for -> yield; for -> exit
             3:2,    # yield -> for;  genexp exit
             5:1,    # list -> exit
-        })
+        }
 
     def test_try_except(self):
         parser = self.parse_source("""\
@@ -72,9 +72,9 @@ class PythonParserTest(CoverageTest):
                 a = 8
             b = 9
             """)
-        self.assertEqual(parser.exit_counts(), {
+        assert parser.exit_counts() == {
             1: 1, 2:1, 3:2, 4:1, 5:2, 6:1, 7:1, 8:1, 9:1
-            })
+            }
 
     def test_excluded_classes(self):
         parser = self.parse_source("""\
@@ -86,9 +86,9 @@ class PythonParserTest(CoverageTest):
                 class Bar:
                     pass
             """)
-        self.assertEqual(parser.exit_counts(), {
+        assert parser.exit_counts() == {
             1:0, 2:1, 3:1
-            })
+            }
 
     def test_missing_branch_to_excluded_code(self):
         parser = self.parse_source("""\
@@ -98,7 +98,7 @@ class PythonParserTest(CoverageTest):
                 a = 4
             b = 5
             """)
-        self.assertEqual(parser.exit_counts(), { 1:1, 2:1, 5:1 })
+        assert parser.exit_counts() == { 1:1, 2:1, 5:1 }
         parser = self.parse_source("""\
             def foo():
                 if fooey:
@@ -107,7 +107,7 @@ class PythonParserTest(CoverageTest):
                     a = 5
             b = 6
             """)
-        self.assertEqual(parser.exit_counts(), { 1:1, 2:2, 3:1, 5:1, 6:1 })
+        assert parser.exit_counts() == { 1:1, 2:2, 3:1, 5:1, 6:1 }
         parser = self.parse_source("""\
             def foo():
                 if fooey:
@@ -116,14 +116,14 @@ class PythonParserTest(CoverageTest):
                     a = 5
             b = 6
             """)
-        self.assertEqual(parser.exit_counts(), { 1:1, 2:1, 3:1, 6:1 })
+        assert parser.exit_counts() == { 1:1, 2:1, 3:1, 6:1 }
 
     def test_indentation_error(self):
         msg = (
             "Couldn't parse '<code>' as Python source: "
             "'unindent does not match any outer indentation level' at line 3"
         )
-        with self.assertRaisesRegex(NotPython, msg):
+        with pytest.raises(NotPython, match=msg):
             _ = self.parse_source("""\
                 0 spaces
                   2
@@ -132,15 +132,14 @@ class PythonParserTest(CoverageTest):
 
     def test_token_error(self):
         msg = "Couldn't parse '<code>' as Python source: 'EOF in multi-line string' at line 1"
-        with self.assertRaisesRegex(NotPython, msg):
+        with pytest.raises(NotPython, match=msg):
             _ = self.parse_source("""\
                 '''
                 """)
 
-
-    @xfail(
-        env.PYPY3 and env.PYPYVERSION == (7, 3, 0),
-        "https://bitbucket.org/pypy/pypy/issues/3139",
+    @pytest.mark.xfail(
+        env.PYPY and env.PYPYVERSION == (7, 3, 0),
+        reason="https://bitbucket.org/pypy/pypy/issues/3139",
     )
     def test_decorator_pragmas(self):
         parser = self.parse_source("""\
@@ -174,8 +173,8 @@ class PythonParserTest(CoverageTest):
         raw_statements = {3, 4, 5, 6, 8, 9, 10, 13, 15, 16, 17, 20, 22, 23, 25, 26}
         if env.PYBEHAVIOR.trace_decorated_def:
             raw_statements.update([11, 19])
-        self.assertEqual(parser.raw_statements, raw_statements)
-        self.assertEqual(parser.statements, {8})
+        assert parser.raw_statements == raw_statements
+        assert parser.statements == {8}
 
     def test_class_decorator_pragmas(self):
         parser = self.parse_source("""\
@@ -188,8 +187,8 @@ class PythonParserTest(CoverageTest):
                 def __init__(self):
                     self.x = 8
             """)
-        self.assertEqual(parser.raw_statements, {1, 2, 3, 5, 6, 7, 8})
-        self.assertEqual(parser.statements, {1, 2, 3})
+        assert parser.raw_statements == {1, 2, 3, 5, 6, 7, 8}
+        assert parser.statements == {1, 2, 3}
 
     def test_empty_decorated_function(self):
         parser = self.parse_source("""\
@@ -219,9 +218,9 @@ class PythonParserTest(CoverageTest):
             expected_arcs.update(set(arcz_to_arcs("-46 6-4")))
             expected_exits.update({6: 1})
 
-        self.assertEqual(expected_statements, parser.statements)
-        self.assertEqual(expected_arcs, parser.arcs())
-        self.assertEqual(expected_exits, parser.exit_counts())
+        assert expected_statements == parser.statements
+        assert expected_arcs == parser.arcs()
+        assert expected_exits == parser.exit_counts()
 
 
 class ParserMissingArcDescriptionTest(CoverageTest):
@@ -237,7 +236,7 @@ class ParserMissingArcDescriptionTest(CoverageTest):
 
     def test_missing_arc_description(self):
         # This code is never run, so the actual values don't matter.
-        parser = self.parse_text(u"""\
+        parser = self.parse_text("""\
             if x:
                 print(2)
             print(3)
@@ -252,34 +251,27 @@ class ParserMissingArcDescriptionTest(CoverageTest):
                     thing(12)
                 more_stuff(13)
             """)
-        self.assertEqual(
-            parser.missing_arc_description(1, 2),
-            "line 1 didn't jump to line 2, because the condition on line 1 was never true"
+        expected = "line 1 didn't jump to line 2, because the condition on line 1 was never true"
+        assert expected == parser.missing_arc_description(1, 2)
+        expected = "line 1 didn't jump to line 3, because the condition on line 1 was never false"
+        assert expected == parser.missing_arc_description(1, 3)
+        expected = (
+            "line 6 didn't return from function 'func5', " +
+            "because the loop on line 6 didn't complete"
         )
-        self.assertEqual(
-            parser.missing_arc_description(1, 3),
-            "line 1 didn't jump to line 3, because the condition on line 1 was never false"
-        )
-        self.assertEqual(
-            parser.missing_arc_description(6, -5),
-            "line 6 didn't return from function 'func5', "
-                            "because the loop on line 6 didn't complete"
-        )
-        self.assertEqual(
-            parser.missing_arc_description(6, 7),
-            "line 6 didn't jump to line 7, because the loop on line 6 never started"
-        )
-        self.assertEqual(
-            parser.missing_arc_description(11, 12),
-            "line 11 didn't jump to line 12, because the condition on line 11 was never true"
-        )
-        self.assertEqual(
-            parser.missing_arc_description(11, 13),
-            "line 11 didn't jump to line 13, because the condition on line 11 was never false"
+        assert expected == parser.missing_arc_description(6, -5)
+        expected = "line 6 didn't jump to line 7, because the loop on line 6 never started"
+        assert expected == parser.missing_arc_description(6, 7)
+        expected = "line 11 didn't jump to line 12, because the condition on line 11 was never true"
+        assert expected == parser.missing_arc_description(11, 12)
+        expected = (
+            "line 11 didn't jump to line 13, " +
+            "because the condition on line 11 was never false"
         )
+        assert expected == parser.missing_arc_description(11, 13)
 
     def test_missing_arc_descriptions_for_small_callables(self):
-        parser = self.parse_text(u"""\
+        parser = self.parse_text("""\
             callables = [
                 lambda: 2,
                 (x for x in range(3)),
@@ -288,25 +280,17 @@ class ParserMissingArcDescriptionTest(CoverageTest):
             ]
             x = 7
             """)
-        self.assertEqual(
-            parser.missing_arc_description(2, -2),
-            "line 2 didn't finish the lambda on line 2"
-        )
-        self.assertEqual(
-            parser.missing_arc_description(3, -3),
-            "line 3 didn't finish the generator expression on line 3"
-        )
-        self.assertEqual(
-            parser.missing_arc_description(4, -4),
-            "line 4 didn't finish the dictionary comprehension on line 4"
-        )
-        self.assertEqual(
-            parser.missing_arc_description(5, -5),
-            "line 5 didn't finish the set comprehension on line 5"
-        )
+        expected = "line 2 didn't finish the lambda on line 2"
+        assert expected == parser.missing_arc_description(2, -2)
+        expected = "line 3 didn't finish the generator expression on line 3"
+        assert expected == parser.missing_arc_description(3, -3)
+        expected = "line 4 didn't finish the dictionary comprehension on line 4"
+        assert expected == parser.missing_arc_description(4, -4)
+        expected = "line 5 didn't finish the set comprehension on line 5"
+        assert expected == parser.missing_arc_description(5, -5)
 
     def test_missing_arc_descriptions_for_exceptions(self):
-        parser = self.parse_text(u"""\
+        parser = self.parse_text("""\
             try:
                 pass
             except ZeroDivideError:
@@ -314,17 +298,19 @@ class ParserMissingArcDescriptionTest(CoverageTest):
             except ValueError:
                 print("yikes")
             """)
-        self.assertEqual(
-            parser.missing_arc_description(3, 4),
-            "line 3 didn't jump to line 4, because the exception caught by line 3 didn't happen"
+        expected = (
+            "line 3 didn't jump to line 4, " +
+            "because the exception caught by line 3 didn't happen"
         )
-        self.assertEqual(
-            parser.missing_arc_description(5, 6),
-            "line 5 didn't jump to line 6, because the exception caught by line 5 didn't happen"
+        assert expected == parser.missing_arc_description(3, 4)
+        expected = (
+            "line 5 didn't jump to line 6, " +
+            "because the exception caught by line 5 didn't happen"
         )
+        assert expected == parser.missing_arc_description(5, 6)
 
     def test_missing_arc_descriptions_for_finally(self):
-        parser = self.parse_text(u"""\
+        parser = self.parse_text("""\
             def function():
                 for i in range(2):
                     try:
@@ -346,59 +332,59 @@ class ParserMissingArcDescriptionTest(CoverageTest):
                 that_thing(19)
             """)
         if env.PYBEHAVIOR.finally_jumps_back:
-            self.assertEqual(
-                parser.missing_arc_description(18, 5),
-                "line 18 didn't jump to line 5, because the break on line 5 wasn't executed"
+            expected = "line 18 didn't jump to line 5, because the break on line 5 wasn't executed"
+            assert expected == parser.missing_arc_description(18, 5)
+            expected = "line 5 didn't jump to line 19, because the break on line 5 wasn't executed"
+            assert expected == parser.missing_arc_description(5, 19)
+            expected = (
+                "line 18 didn't jump to line 10, " +
+                "because the continue on line 10 wasn't executed"
             )
-            self.assertEqual(
-                parser.missing_arc_description(5, 19),
-                "line 5 didn't jump to line 19, because the break on line 5 wasn't executed"
+            assert expected == parser.missing_arc_description(18, 10)
+            expected = (
+                "line 10 didn't jump to line 2, " +
+                "because the continue on line 10 wasn't executed"
             )
-            self.assertEqual(
-                parser.missing_arc_description(18, 10),
-                "line 18 didn't jump to line 10, because the continue on line 10 wasn't executed"
+            assert expected == parser.missing_arc_description(10, 2)
+            expected = (
+                "line 18 didn't jump to line 14, " +
+                "because the return on line 14 wasn't executed"
             )
-            self.assertEqual(
-                parser.missing_arc_description(10, 2),
-                "line 10 didn't jump to line 2, because the continue on line 10 wasn't executed"
+            assert expected == parser.missing_arc_description(18, 14)
+            expected = (
+                "line 14 didn't return from function 'function', " +
+                "because the return on line 14 wasn't executed"
             )
-            self.assertEqual(
-                parser.missing_arc_description(18, 14),
-                "line 18 didn't jump to line 14, because the return on line 14 wasn't executed"
-            )
-            self.assertEqual(
-                parser.missing_arc_description(14, -1),
-                "line 14 didn't return from function 'function', "
-                    "because the return on line 14 wasn't executed"
-            )
-            self.assertEqual(
-                parser.missing_arc_description(18, -1),
-                "line 18 didn't except from function 'function', "
-                    "because the raise on line 16 wasn't executed"
+            assert expected == parser.missing_arc_description(14, -1)
+            expected = (
+                "line 18 didn't except from function 'function', " +
+                "because the raise on line 16 wasn't executed"
             )
+            assert expected == parser.missing_arc_description(18, -1)
         else:
-            self.assertEqual(
-                parser.missing_arc_description(18, 19),
-                "line 18 didn't jump to line 19, because the break on line 5 wasn't executed"
+            expected = (
+                "line 18 didn't jump to line 19, " +
+                "because the break on line 5 wasn't executed"
             )
-            self.assertEqual(
-                parser.missing_arc_description(18, 2),
-                "line 18 didn't jump to line 2, "
-                    "because the continue on line 10 wasn't executed"
-                    " or "
+            assert expected == parser.missing_arc_description(18, 19)
+            expected = (
+                "line 18 didn't jump to line 2, " +
+                    "because the continue on line 10 wasn't executed" +
+                " or " +
                     "the continue on line 12 wasn't executed"
             )
-            self.assertEqual(
-                parser.missing_arc_description(18, -1),
-                "line 18 didn't except from function 'function', "
-                    "because the raise on line 16 wasn't executed"
-                " or "
-                "line 18 didn't return from function 'function', "
+            assert expected == parser.missing_arc_description(18, 2)
+            expected = (
+                "line 18 didn't except from function 'function', " +
+                    "because the raise on line 16 wasn't executed" +
+                " or " +
+                "line 18 didn't return from function 'function', " +
                     "because the return on line 14 wasn't executed"
             )
+            assert expected == parser.missing_arc_description(18, -1)
 
     def test_missing_arc_descriptions_bug460(self):
-        parser = self.parse_text(u"""\
+        parser = self.parse_text("""\
             x = 1
             d = {
                 3: lambda: [],
@@ -406,10 +392,25 @@ class ParserMissingArcDescriptionTest(CoverageTest):
             }
             x = 6
             """)
-        self.assertEqual(
-            parser.missing_arc_description(2, -3),
-            "line 3 didn't finish the lambda on line 3",
-        )
+        assert parser.missing_arc_description(2, -3) == "line 3 didn't finish the lambda on line 3"
+
+    @pytest.mark.skipif(not env.PYBEHAVIOR.match_case, reason="Match-case is new in 3.10")
+    def test_match_case_with_default(self):
+        parser = self.parse_text("""\
+            for command in ["huh", "go home", "go n"]:
+                match command.split():
+                    case ["go", direction] if direction in "nesw":
+                        match = f"go: {direction}"
+                    case ["go", _]:
+                        match = "no go"
+                print(match)
+            """)
+        assert parser.missing_arc_description(3, 4) == (
+            "line 3 didn't jump to line 4, because the pattern on line 3 never matched"
+            )
+        assert parser.missing_arc_description(3, 5) == (
+            "line 3 didn't jump to line 5, because the pattern on line 3 always matched"
+            )
 
 
 class ParserFileTest(CoverageTest):
@@ -440,18 +441,14 @@ class ParserFileTest(CoverageTest):
             fname = fname + ".py"
             self.make_file(fname, text, newline=newline)
             parser = self.parse_file(fname)
-            self.assertEqual(
-                parser.exit_counts(),
-                counts,
-                "Wrong for %r" % fname
-            )
+            assert parser.exit_counts() == counts, "Wrong for %r" % fname
 
     def test_encoding(self):
         self.make_file("encoded.py", """\
             coverage = "\xe7\xf6v\xear\xe3g\xe9"
             """)
         parser = self.parse_file("encoded.py")
-        self.assertEqual(parser.exit_counts(), {1: 1})
+        assert parser.exit_counts() == {1: 1}
 
     def test_missing_line_ending(self):
         # Test that the set of statements is the same even if a final
@@ -466,7 +463,7 @@ class ParserFileTest(CoverageTest):
             """)
 
         parser = self.parse_file("normal.py")
-        self.assertEqual(parser.statements, {1})
+        assert parser.statements == {1}
 
         self.make_file("abrupt.py", """\
             out, err = subprocess.Popen(
@@ -476,7 +473,7 @@ class ParserFileTest(CoverageTest):
 
         # Double-check that some test helper wasn't being helpful.
         with open("abrupt.py") as f:
-            self.assertEqual(f.read()[-1], ")")
+            assert f.read()[-1] == ")"
 
         parser = self.parse_file("abrupt.py")
-        self.assertEqual(parser.statements, {1})
+        assert parser.statements == {1}
